@@ -27,7 +27,7 @@ from localstack.services.infra import PROXY_LISTENERS
 from localstack.services.plugins import SERVICE_PLUGINS
 from localstack.services.s3.s3_utils import uses_host_addressing
 from localstack.services.sqs.sqs_listener import is_sqs_queue_url
-from localstack.utils import persistence
+from localstack.utils import common, persistence
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_stack import is_internal_call_context, set_default_region_in_headers
 from localstack.utils.aws.request_routing import extract_version_and_action, matches_service_action
@@ -74,6 +74,9 @@ class ProxyListenerEdge(ProxyListener):
 
     def forward_request(self, method, path, data, headers):
 
+        if common.INFRA_STOPPED:
+            return 503
+
         if config.EDGE_FORWARD_URL:
             return do_forward_request_network(
                 0, method, path, data, headers, target_url=config.EDGE_FORWARD_URL
@@ -445,6 +448,9 @@ def get_api_from_custom_rules(method, path, data, headers):
     if _in_path_or_payload("Action=AssumeRoleWithSAML"):
         return "sts", config.service_port("sts")
 
+    if _in_path_or_payload("Action=AssumeRole"):
+        return "sts", config.service_port("sts")
+
     # CloudWatch backdoor API to retrieve raw metrics
     if path.startswith(PATH_GET_RAW_METRICS):
         return "cloudwatch", config.service_port("cloudwatch")
@@ -651,7 +657,8 @@ def run_process_as_sudo(component, port, asynchronous=False, env_vars=None):
         sudo_cmd,
         env_vars_str,
         python_cmd,
-        __file__,
+        "-m",
+        "localstack.services.edge",
         component,
         str(port),
     ]
