@@ -1,11 +1,12 @@
 from requests.models import Request
+
 from localstack.services.generic_proxy import ProxyListener
 from localstack.utils.aws.aws_responses import MessageConversion
 
 
 class ProxyListenerRG(ProxyListener):
     def forward_request(self, method, path, data, headers):
-        if method == 'POST' and path == '/':
+        if method == "POST" and path == "/":
             data = MessageConversion._reset_account_id(data)
             return Request(data=data, headers=headers, method=method)
 
@@ -20,7 +21,7 @@ class ProxyListenerRG(ProxyListener):
             # fix returned error codes
             MessageConversion.fix_error_codes(method, data, response)
             # fix content-length header
-            response.headers['Content-Length'] = str(len(response._content))
+            response.headers["Content-Length"] = str(len(response._content))
 
 
 # instantiate listener
