@@ -26,6 +26,8 @@ EVENT_DYNAMODB_CREATE_STREAM = "ddb.cs"
 EVENT_CLOUDFORMATION_CREATE_STACK = "clf.cs"
 EVENT_ES_CREATE_DOMAIN = "es.cd"
 EVENT_ES_DELETE_DOMAIN = "es.dd"
+EVENT_OPENSEARCH_CREATE_DOMAIN = "os.cd"
+EVENT_OPENSEARCH_DELETE_DOMAIN = "os.dd"
 EVENT_FIREHOSE_CREATE_STREAM = "fho.cs"
 EVENT_FIREHOSE_DELETE_STREAM = "fho.ds"
 
