@@ -1,8 +1,7 @@
 import base64
 import json
 import re
-
-from six.moves.urllib.parse import quote_plus, unquote_plus
+from urllib.parse import quote_plus, unquote_plus
 
 from localstack import config
 from localstack.utils.common import (
@@ -80,7 +79,9 @@ class VelocityUtil(object):
         return json.dumps(s)
 
 
-def render_velocity_template(template, context, variables={}, as_json=False):
+def render_velocity_template(template, context, variables=None, as_json=False):
+    if variables is None:
+        variables = {}
     import airspeed
 
     if not template:
