@@ -33,7 +33,6 @@ from urllib.parse import parse_qs, urlparse
 import cachetools
 import dns.resolver
 import requests
-import six
 from requests import Response
 from requests.models import CaseInsensitiveDict
 
@@ -137,7 +136,7 @@ class CustomEncoder(json.JSONEncoder):
                 return bool(o.value)
             return str(o.value)
         try:
-            if isinstance(o, six.binary_type):
+            if isinstance(o, bytes):
                 return to_str(o)
             return super(CustomEncoder, self).default(o)
         except Exception:
@@ -486,6 +485,45 @@ class HashableList(list):
         return result
 
 
+class PaginatedList(list):
+    """List which can be paginated and filtered. For usage in AWS APIs with paginated responses"""
+
+    DEFAULT_PAGE_SIZE = 50
+
+    def get_page(
+        self,
+        token_generator: Callable,
+        next_token: str = None,
+        page_size: int = None,
+        filter_function: Callable = None,
+    ) -> (list, str):
+        if filter_function is not None:
+            result_list = list(filter(filter_function, self))
+        else:
+            result_list = self
+
+        if page_size is None:
+            page_size = self.DEFAULT_PAGE_SIZE
+
+        if len(result_list) <= page_size:
+            return result_list, None
+
+        start_idx = 0
+
+        try:
+            start_item = next(item for item in result_list if token_generator(item) == next_token)
+            start_idx = result_list.index(start_item)
+        except StopIteration:
+            pass
+
+        if start_idx + page_size <= len(result_list):
+            next_token = token_generator(result_list[start_idx + page_size])
+        else:
+            next_token = None
+
+        return result_list[start_idx : start_idx + page_size], next_token
+
+
 class FileMappedDocument(dict):
     """A dictionary that is mapped to a json document on disk.
 
@@ -666,17 +704,17 @@ def prevent_stack_overflow(match_parameters=False):
 
 
 def is_string(s, include_unicode=True, exclude_binary=False):
-    if isinstance(s, six.binary_type) and exclude_binary:
+    if isinstance(s, bytes) and exclude_binary:
         return False
     if isinstance(s, str):
         return True
-    if include_unicode and isinstance(s, six.text_type):
+    if include_unicode and isinstance(s, str):
         return True
     return False
 
 
 def is_string_or_bytes(s):
-    return is_string(s) or isinstance(s, six.string_types) or isinstance(s, bytes)
+    return is_string(s) or isinstance(s, str) or isinstance(s, bytes)
 
 
 def is_base64(s):
@@ -758,7 +796,7 @@ def is_port_open(
     host = "localhost"
     protocol = "http"
     protocols = protocols if isinstance(protocols, list) else [protocols]
-    if isinstance(port, six.string_types):
+    if isinstance(port, str):
         url = urlparse(port_or_url)
         port = url.port
         host = url.hostname
@@ -908,7 +946,7 @@ def to_unique_items_list(inputs, comparator=None):
 def timestamp(time=None, format: str = TIMESTAMP_FORMAT) -> str:
     if not time:
         time = datetime.utcnow()
-    if isinstance(time, six.integer_types + (float,)):
+    if isinstance(time, (int, float)):
         time = datetime.fromtimestamp(time)
     return time.strftime(format)
 
@@ -1003,6 +1041,10 @@ def merge_dicts(*dicts, **kwargs):
     return result
 
 
+def remove_none_values_from_dict(dict: Dict) -> Dict:
+    return {k: v for (k, v) in dict.items() if v is not None}
+
+
 def recurse_object(obj: JsonType, func: Callable, path: str = "") -> Any:
     """Recursively apply `func` to `obj` (may be a list, dict, or other object)."""
     obj = func(obj, path=path)
@@ -1546,7 +1588,7 @@ def extract_from_jsonpointer_path(target, path: str, delimiter: str = "/", auto_
 
 def save_file(file, content, append=False, permissions=None):
     mode = "a" if append else "w+"
-    if not isinstance(content, six.string_types):
+    if not isinstance(content, str):
         mode = mode + "b"
 
     def _opener(path, flags):
@@ -1592,13 +1634,13 @@ def replace_in_file(search, replace, file_path):
 def to_str(obj: Union[str, bytes], encoding: str = DEFAULT_ENCODING, errors="strict") -> str:
     """If ``obj`` is an instance of ``binary_type``, return
     ``obj.decode(encoding, errors)``, otherwise return ``obj``"""
-    return obj.decode(encoding, errors) if isinstance(obj, six.binary_type) else obj
+    return obj.decode(encoding, errors) if isinstance(obj, bytes) else obj
 
 
 def to_bytes(obj: Union[str, bytes], encoding: str = DEFAULT_ENCODING, errors="strict") -> bytes:
     """If ``obj`` is an instance of ``text_type``, return
     ``obj.encode(encoding, errors)``, otherwise return ``obj``"""
-    return obj.encode(encoding, errors) if isinstance(obj, six.text_type) else obj
+    return obj.encode(encoding, errors) if isinstance(obj, str) else obj
 
 
 def str_to_bool(value):
@@ -2083,7 +2125,7 @@ class NetrcBypassAuth(requests.auth.AuthBase):
         return r
 
 
-class _RequestsSafe(type):
+class _RequestsSafe:
     """Wrapper around requests library, which can prevent it from verifying
     SSL certificates or reading credentials from ~/.netrc file"""
 
@@ -2105,6 +2147,10 @@ class _RequestsSafe(type):
         return _wrapper
 
 
+# create safe_requests instance
+safe_requests = _RequestsSafe()
+
+
 class FileListener:
     """
     Platform independent `tail -f` command that calls a callback every time a new line is received on the file. If
@@ -2185,11 +2231,6 @@ class FileListener:
         return FuncThread(func=_run_follow, on_stop=lambda *_: tailer.close())
 
 
-# create class-of-a-class
-class safe_requests(six.with_metaclass(_RequestsSafe)):
-    pass
-
-
 def make_http_request(
     url: str, data: Union[bytes, str] = None, headers: Dict[str, str] = None, method: str = "GET"
 ) -> Response:
@@ -2198,15 +2239,6 @@ def make_http_request(
     )
 
 
-class SafeStringIO(io.StringIO):
-    """Safe StringIO implementation that doesn't fail if str is passed in Python 2."""
-
-    def write(self, obj):
-        if six.PY2 and isinstance(obj, str):
-            obj = obj.decode("unicode-escape")
-        return super(SafeStringIO, self).write(obj)
-
-
 def clean_cache(file_pattern=CACHE_FILE_PATTERN, last_clean_time=None, max_age=CACHE_MAX_AGE):
     if last_clean_time is None:
         last_clean_time = last_cache_clean_time
